package cn.com.infosec.x509;

import cn.com.infosec.asn1.ASN1Encodable;
import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.DERBitString;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.x509.AttributeCertificate;
import cn.com.infosec.asn1.x509.X509Extension;
import cn.com.infosec.asn1.x509.X509Extensions;
import cn.com.infosec.util.Arrays;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Set;

public class X509V2AttributeCertificate implements X509AttributeCertificate {
   private AttributeCertificate cert;
   private Date notBefore;
   private Date notAfter;

   public X509V2AttributeCertificate(InputStream encIn) {
      this(AttributeCertificate.getInstance((new ASN1InputStream(encIn)).readObject()));
   }

   public X509V2AttributeCertificate(byte[] encoded) {
      X509V2AttributeCertificate var10000 = this;
      ByteArrayInputStream var2;
      var2 = new ByteArrayInputStream.<init>(encoded);
      var10000.<init>((InputStream)var2);
   }

   X509V2AttributeCertificate(AttributeCertificate cert) {
      X509V2AttributeCertificate var10000 = this;
      AttributeCertificate var10001 = cert;
      X509V2AttributeCertificate var10002 = this;
      AttributeCertificate var10003 = cert;
      super();
      this.cert = cert;

      try {
         var10002.notAfter = var10003.getAcinfo().getAttrCertValidityPeriod().getNotAfterTime().getDate();
         var10000.notBefore = var10001.getAcinfo().getAttrCertValidityPeriod().getNotBeforeTime().getDate();
      } catch (ParseException var2) {
         throw new IOException("invalid data structure in certificate!");
      }
   }

   private Set getExtensionOIDs(boolean critical) {
      X509Extensions var5;
      if ((var5 = this.cert.getAcinfo().getExtensions()) != null) {
         HashSet var2;
         var2 = new HashSet.<init>();
         Enumeration var3 = var5.oids();

         while(var3.hasMoreElements()) {
            DERObjectIdentifier var4;
            if (var5.getExtension(var4 = (DERObjectIdentifier)var3.nextElement()).isCritical() == critical) {
               var2.add(var4.getId());
            }
         }

         return var2;
      } else {
         return null;
      }
   }

   public int getVersion() {
      return this.cert.getAcinfo().getVersion().getValue().intValue();
   }

   public BigInteger getSerialNumber() {
      return this.cert.getAcinfo().getSerialNumber().getValue();
   }

   public AttributeCertificateHolder getHolder() {
      return new AttributeCertificateHolder((ASN1Sequence)this.cert.getAcinfo().getHolder().toASN1Object());
   }

   public AttributeCertificateIssuer getIssuer() {
      return new AttributeCertificateIssuer(this.cert.getAcinfo().getIssuer());
   }

   public Date getNotBefore() {
      return this.notBefore;
   }

   public Date getNotAfter() {
      return this.notAfter;
   }

   public boolean[] getIssuerUniqueID() {
      DERBitString var5;
      if ((var5 = this.cert.getAcinfo().getIssuerUniqueID()) != null) {
         byte[] var1;
         int var6;
         boolean[] var2 = new boolean[var6 = (var1 = var5.getBytes()).length * 8 - var5.getPadBits()];

         for(int var3 = 0; var3 != var6; ++var3) {
            boolean var4;
            if ((var1[var3 / 8] & 128 >>> var3 % 8) != 0) {
               var4 = true;
            } else {
               var4 = false;
            }

            var2[var3] = var4;
         }

         return var2;
      } else {
         return null;
      }
   }

   public void checkValidity() {
      X509V2AttributeCertificate var10000 = this;
      Date var1;
      var1 = new Date.<init>();
      var10000.checkValidity(var1);
   }

   public void checkValidity(Date date) {
      if (!date.after(this.getNotAfter())) {
         if (date.before(this.getNotBefore())) {
            throw new CertificateNotYetValidException("certificate not valid till " + this.getNotBefore());
         }
      } else {
         throw new CertificateExpiredException("certificate expired on " + this.getNotAfter());
      }
   }

   public byte[] getSignature() {
      return this.cert.getSignatureValue().getBytes();
   }

   public final void verify(PublicKey key, String provider) {
      if (this.cert.getSignatureAlgorithm().equals(this.cert.getAcinfo().getSignature())) {
         Signature provider1;
         Signature var10000 = provider1 = Signature.getInstance(this.cert.getSignatureAlgorithm().getObjectId().getId(), provider);
         Signature var10002 = provider1;
         X509V2AttributeCertificate var10003 = this;
         provider1.initVerify(key);

         try {
            var10002.update(var10003.cert.getAcinfo().getEncoded());
         } catch (IOException var3) {
            throw new SignatureException("Exception encoding certificate info object");
         }

         if (!var10000.verify(this.getSignature())) {
            throw new InvalidKeyException("Public key presented not for certificate signature");
         }
      } else {
         throw new CertificateException("Signature algorithm in certificate info not same as outer certificate");
      }
   }

   public byte[] getEncoded() {
      return this.cert.getEncoded();
   }

   public byte[] getExtensionValue(String oid) {
      X509Extensions var3;
      if ((var3 = this.cert.getAcinfo().getExtensions()) != null) {
         X509Extensions var10000 = var3;
         DERObjectIdentifier var4;
         var4 = new DERObjectIdentifier.<init>(oid);
         X509Extension var5;
         if ((var5 = var10000.getExtension(var4)) != null) {
            try {
               return var5.getValue().getEncoded("DER");
            } catch (Exception var2) {
               throw new RuntimeException("error encoding " + var2.toString());
            }
         }
      }

      return null;
   }

   public Set getNonCriticalExtensionOIDs() {
      return this.getExtensionOIDs(false);
   }

   public Set getCriticalExtensionOIDs() {
      return this.getExtensionOIDs(true);
   }

   public boolean hasUnsupportedCriticalExtension() {
      Set var1;
      return (var1 = this.getCriticalExtensionOIDs()) != null && !var1.isEmpty();
   }

   public X509Attribute[] getAttributes() {
      ASN1Sequence var4;
      X509Attribute[] var1 = new X509Attribute[(var4 = this.cert.getAcinfo().getAttributes()).size()];

      for(int var2 = 0; var2 != var4.size(); ++var2) {
         X509Attribute var3;
         var3 = new X509Attribute.<init>((ASN1Encodable)var4.getObjectAt(var2));
         var1[var2] = var3;
      }

      return var1;
   }

   public X509Attribute[] getAttributes(String oid) {
      ASN1Sequence var5 = this.cert.getAcinfo().getAttributes();
      ArrayList var2;
      var2 = new ArrayList.<init>();

      for(int var3 = 0; var3 != var5.size(); ++var3) {
         X509Attribute var4;
         X509Attribute var10000 = var4 = new X509Attribute;
         var10000.<init>((ASN1Encodable)var5.getObjectAt(var3));
         if (var10000.getOID().equals(oid)) {
            var2.add(var4);
         }
      }

      if (var2.size() == 0) {
         return null;
      } else {
         return (X509Attribute[])var2.toArray(new X509Attribute[var2.size()]);
      }
   }

   public boolean equals(Object o) {
      if (o == this) {
         return true;
      } else if (!(o instanceof X509AttributeCertificate)) {
         return false;
      } else {
         X509V2AttributeCertificate var10000 = this;
         X509AttributeCertificate var3 = (X509AttributeCertificate)o;

         try {
            return Arrays.areEqual(var10000.getEncoded(), var3.getEncoded());
         } catch (IOException var2) {
            return false;
         }
      }
   }

   public int hashCode() {
      byte[] var10000;
      boolean var10001;
      try {
         var10000 = this.getEncoded();
      } catch (IOException var6) {
         var10001 = false;
         return 0;
      }

      byte[] var7 = var10000;
      int var1 = 0;
      int var2 = 0;

      while(true) {
         int var8;
         int var9;
         try {
            var8 = var2;
            var9 = var7.length;
         } catch (IOException var5) {
            var10001 = false;
            break;
         }

         if (var8 == var9) {
            return var1;
         }

         byte var10;
         try {
            var8 = var1;
            var10 = var7[var2];
         } catch (IOException var4) {
            var10001 = false;
            break;
         }

         int var10002;
         try {
            var10002 = var2 % 4;
         } catch (IOException var3) {
            var10001 = false;
            break;
         }

         var1 = var8 ^ (var10 & 255) << var10002;
         ++var2;
      }

      return 0;
   }
}
